"use strict";
var JPromise = Promise || require("bluebird");

/**
 * A Jefferson proxy that resolves promise-based middleware functions.
 * If a middleware function accepts 2 arguments, we will wrap it in a promise chain.
 * If it throws or resolves to an error, next() is invoked with the error.
 *
 * NOTE: This should be the innermost proxy
 *
 * @type {{name: string, init: Function}}
 */
module.exports = {
    name: "Promise Handler",
    init: function (delegate, conf, middlewareIndex) {
        if (!delegate || typeof delegate !== "function") {
            throw new Error("'delegate' argument must exist and be a function. MiddlewareIndex: " + middlewareIndex);
        }
        return function (req, res, next) {
            var nextTriggered = false;
            var invokeNext = function (arg) {
                if (!nextTriggered) {
                    nextTriggered = true;
                    next(arg);
                }
            };
            var isFlowHalted = function () {
                return conf && conf.haltCondition && conf.haltCondition(req, res);
            };
            var possiblePromise = delegate(req, res, invokeNext);
            return JPromise.resolve(possiblePromise).then(function () {
                if (!isFlowHalted()) {
                    invokeNext();
                }
            })["catch"](next);
        };
    },

    /**
     * Promisifies an async middleware function.
     * @param delegate
     */
    promisify: function (delegate) {
        return function (req, res, next) {
            return new JPromise(function (resolve, reject) {
                var proxyNext = function (err) {
                    next(err);
                    if (err) {
                        reject(err);
                    } else {
                        resolve();
                    }
                };
                delegate(req, res, proxyNext);
            });
        };
    }
};